//	Explosion.cpp - Class handling explosion effects.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "Explosion.h"
#include "Random.h"

//------------------------------------------------------------------------------
Explosion::Explosion(const TwoFloats& pos, const ThreeFloats& col):
dead(false)
{
	shared_ptr<FancyImage> im;

	//Star flash.
	im = shared_ptr<FancyImage>(new FancyImage());
	im->setImage(L"StarFlash");
	im->setPosition(pos);
	im->setScale(0.0f);
	im->tweenScale(1.0f, 0.02f);
	im->tweenAlpha(0.0f, 0.05f, FancyImage::Logarithm, 0.33f);
	im->setPivot(192.0f);
	im->addStepRotation(-2.0f * NIALL_PI, 0.001f, FancyImage::Linear, 0.0f);
	images.push_back(im);

	im = shared_ptr<FancyImage>(new FancyImage());
	im->setImage(L"StarFlash");
	im->setPosition(pos);
	im->setScale(0.0f);
	im->tweenScale(1.0f, 0.02f);
	im->tweenAlpha(0.0f, 0.05f, FancyImage::Logarithm, 0.33f);
	im->setPivot(192.0f);
	im->setRotation(0.125f * NIALL_PI);
	im->addStepRotation(-2.0f * NIALL_PI, 0.001f, FancyImage::Linear, 0.0f);
	images.push_back(im);

	//Clouds.
	for(int i=0;i<8;++i)
	{
		im = shared_ptr<FancyImage>(new FancyImage());
		im->setImage(L"Cloud");
		im->setPosition(pos);
		im->tweenPosition(TwoFloats(pos.x + (RandFloat(128.0f) - 64.0f),
									pos.y + (RandFloat(128.0f) - 64.0f)));
		im->setScale(0.0f);
		im->tweenScale(1.0f, 0.01f + RandFloat(0.02f), FancyImage::Logarithm);
		im->setColour(col + (RandFloat(0.2f) - 0.1f));
		im->setAlpha(0.0f);
		im->addStepAlpha(0.66f, 0.1f, FancyImage::Logarithm);
		im->addStepAlpha(0.0f, 0.02f, FancyImage::Linear, 0.5f);
		im->setPivot(TwoFloats(28.0f, 22.0f));
		im->setRotation(RandFloat(2.0f * NIALL_PI));
		im->addStepRotation(RandFloat(2.0f * NIALL_PI), 0.01f, FancyImage::Linear, 0.0f, false, true);
		images.push_back(im);
	}

	for(int i=0;i<16;++i)
	{
		im = shared_ptr<FancyImage>(new FancyImage());
		im->setImage(L"Particle");
		im->setPosition(pos);
		im->tweenPosition(TwoFloats(pos.x + (RandFloat(128.0f) - 64.0f),
									pos.y + (RandFloat(128.0f) - 64.0f)));
		im->setScale(0.0f);
		im->tweenScale(1.0f, 0.01f + RandFloat(0.02f), FancyImage::Logarithm);
		im->setColour(col + (RandFloat(0.2f)));
		im->setAlpha(0.0f);
		im->addStepAlpha(0.66f, 0.1f, FancyImage::Logarithm);
		im->addStepAlpha(0.0f, 0.02f, FancyImage::Linear, 0.5f);
		im->setPivot(TwoFloats(6.0f, 10.0f));
		im->setRotation(RandFloat(2.0f * NIALL_PI));
		im->addStepRotation(RandFloat(2.0f * NIALL_PI), 0.01f, FancyImage::Linear, 0.0f, false, true);
		images.push_back(im);
	}
}

//------------------------------------------------------------------------------
void Explosion::update()
{
	int numDead(0);

	for(unsigned int i=0;i<images.size();++i)
	{
		if(!images[i]->getFinished())
			images[i]->update();
		else
			++numDead;
	}
}

//------------------------------------------------------------------------------
void Explosion::draw(shared_ptr<Drawer>& d)
{
	for(unsigned int i=0;i<images.size();++i)
	{
		if(!images[i]->getFinished())
			images[i]->draw(d);
	}
}
