//	Explosion.h - Class handling explosion effects.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef EXPLOSION_H_
#define EXPLOSION_H_

#include "drawing/FancyImage.h"
#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"

#include <vector>

class Drawer;

///	Class handling explosion effects.
class Explosion
{
  public:
	///	Constructor.
	Explosion(const TwoFloats& pos, const ThreeFloats& col);

	///	Updates the explosion.
	void update();
	///	Draws the explosion.
	void draw(shared_ptr<Drawer>& d);

	///	Returns true when the explosion has finished.
	bool getDead() const {return dead;};
  private:
	///	The FancyImages which make up the explosion.
	std::vector<shared_ptr<FancyImage> > images;
	///	True if the explosion has finished.
	bool dead;
};

#endif
