//	NiallArray.h - A fixed-size array class with operators for addition etc.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef NIALLARRAY_H_
#define NIALLARRAY_H_

///	A fixed-size array class with operators for addition etc.
/*!
	This is so I can use Tweener to tween arrays of data easily. Sadly
	std::array doesn't include the various operators I would need to do this.
 */
template <typename T, int N>
class NiallArray
{
  public:
	///	Default constructor.
	NiallArray() {};
	///	Copy constructor.
	NiallArray(const NiallArray<T, N>& other)
	{
		for(int i=0;i<N;++i)
			arr[i] = other.arr[i];
	};

	///	Access operator.
	const T operator[](int index) const {return arr[index];};
	///	Access assignment operator.
	T& operator[](int index) {return arr[index];};

	///	Assignment operator.
	NiallArray<T, N>& operator=(const NiallArray<T, N>& other)
	{
		for(int i=0;i<N;++i)
			arr[i] = other.arr[i];

		return *this;
	};
	///	Addition operator.
	NiallArray<T, N> operator+(const NiallArray<T, N>& other)
	{
		NiallArray<T, N> retval;

		for(int i=0;i<N;++i)
			retval[i] = arr[i] + other.arr[i];

		return retval;
	};
	///	Substraction operator.
	NiallArray<T, N> operator-(const NiallArray<T, N>& other)
	{
		NiallArray<T, N> retval;

		for(int i=0;i<N;++i)
			retval[i] = arr[i] - other.arr[i];

		return retval;
	};
	///	Multiplication operator.
	NiallArray<T, N> operator*(const NiallArray<T, N>& other)
	{
		NiallArray<T, N> retval;

		for(int i=0;i<N;++i)
			retval[i] = arr[i] * other.arr[i];

		return retval;
	};
	///	Division operator.
	NiallArray<T, N> operator/(const NiallArray<T, N>& other)
	{
		NiallArray<T, N> retval;

		for(int i=0;i<N;++i)
			retval[i] = arr[i] / other.arr[i];

		return retval;
	};

	///	Assignment operator (float).
	NiallArray<T, N>& operator=(float other)
	{
		for(int i=0;i<N;++i)
			arr[i] = other;

		return *this;
	};
	///	Addition operator (float).
	NiallArray<T, N> operator+(float other)
	{
		NiallArray<T, N> retval;

		for(int i=0;i<N;++i)
			retval[i] = arr[i] + other;

		return retval;
	};
	///	Substraction operator (float).
	NiallArray<T, N> operator-(float other)
	{
		NiallArray<T, N> retval;

		for(int i=0;i<N;++i)
			retval[i] = arr[i] - other;

		return retval;
	};
	///	Multiplication operator (float).
	NiallArray<T, N> operator*(float other)
	{
		NiallArray<T, N> retval;

		for(int i=0;i<N;++i)
			retval[i] = arr[i] * other;

		return retval;
	};
	///	Division operator (float).
	NiallArray<T, N> operator/(float other)
	{
		NiallArray<T, N> retval;

		for(int i=0;i<N;++i)
			retval[i] = arr[i] / other;

		return retval;
	};
  private:
	///	The actual array.
	T arr[N];
};

#endif
