//	TitleScreen.cpp - The title screen.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/FancyImage.h"
#include "WakeUpGlobalData.h"
#include "drawing/Drawer.h"
#include "ColourHelpers.h"
#include "WakeUpSounder.h"
#include "TitleScreen.h"
#include "Application.h"
#include "MiscHelper.h"
#include "Explosion.h"
#include "Entity.h"
#include "Random.h"

#include <sstream>
#include <iomanip>

//------------------------------------------------------------------------------
GameStateCreator<TitleScreen> TitleScreen::creator;

//------------------------------------------------------------------------------
TitleScreen::TitleScreen(Application& app, SounderPtr& sound):
GameState(app, sound),
mousePos(640.0f, 360.0f),
filteredPos(640.0f, 360.0f),
imageIndex(0),
entityCount(180),
currentEntry(0),
firstGo(true),
joyCount(0),
fade(1.0f),
fadeTween(fade, 0.0f, 0.01f),
updateFromDraw(false),
shakeAmp(0.0f),
shakeIndex(0.0f),
sounder(dynamic_pointer_cast<WakeUpSounder>(sound))
{
	const TwoFloats& drawSize(app.getDrawingAreaSize());

	app.setMousePos(mousePos);

	highlights[0].setImage(L"StarFlash");
	highlights[0].setPosition(TwoFloats((drawSize.x * 0.5f) - 320.0f,
										(drawSize.y * 0.5f)));
	highlights[0].setScale(0.0f);
	highlights[0].tweenScale(1.0f, 0.05f);
	highlights[0].setPivot(192.0f);
	highlights[0].addStepRotation(-2.0f * NIALL_PI,
								  0.001f,
								  FancyImage::Linear,
								  0.0f,
								  true,
								  true);
	highlights[0].addStepScale(0.8f, 0.005f, FancyImage::Smoothstep, 0.0f, true);
	highlights[0].addStepScale(1.0f, 0.005f, FancyImage::Smoothstep, 0.0f, true);
	highlights[0].setColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)));
	highlights[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);

	highlights[1].setImage(L"StarFlash");
	highlights[1].setPosition(TwoFloats((drawSize.x * 0.5f) - 320.0f,
										(drawSize.y * 0.5f)));
	highlights[1].setScale(0.0f);
	highlights[1].tweenScale(1.0f, 0.05f, FancyImage::Linear, 0.0f);
	highlights[1].setPivot(192.0f);
	highlights[1].setRotation(0.125f * NIALL_PI);
	highlights[1].addStepRotation(-2.0f * NIALL_PI,
								  0.001f,
								  FancyImage::Linear,
								  0.0f,
								  true,
								  true);
	highlights[1].addStepScale(1.0f, 0.005f, FancyImage::Smoothstep, 0.0f, true);
	highlights[1].addStepScale(0.8f, 0.005f, FancyImage::Smoothstep, 0.0f, true);
	highlights[1].setColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)));
	highlights[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	highlights[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.75f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);


	clockImages[0].setImage(L"ClockBack");
	clockImages[0].setPosition(TwoFloats((drawSize.x * 0.5f) - 320.0f,
										 (drawSize.y * 0.5f)));
	clockImages[0].setColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)));
	clockImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);

	clockImages[1].setImage(L"ClockHand");
	clockImages[1].setPosition(TwoFloats((drawSize.x * 0.5f) - 320.0f,
										 (drawSize.y * 0.5f)));
	clockImages[1].setColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)));
	clockImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	clockImages[1].setPivot(96.0f);
	clockImages[1].addStepRotation(2.0f * NIALL_PI, 0.001f, FancyImage::Linear, 0.0f, true, true);

	playgroundImages[0].setImage(L"SwingBack");
	playgroundImages[0].setPosition(TwoFloats((drawSize.x * 0.5f) + 320.0f,
											  (drawSize.y * 0.5f)));
	playgroundImages[0].setColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)));
	playgroundImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[0].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);

	playgroundImages[1].setImage(L"SwingSeat");
	playgroundImages[1].setPosition(TwoFloats((drawSize.x * 0.5f) + 320.0f,
											  (drawSize.y * 0.5f)));
	playgroundImages[1].setColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)));
	playgroundImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[1].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[1].setPivot(TwoFloats(99.0f, 14.0f));

	playgroundImages[2].setImage(L"SwingFront");
	playgroundImages[2].setPosition(TwoFloats((drawSize.x * 0.5f) + 320.0f,
											  (drawSize.y * 0.5f)));
	playgroundImages[2].setColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)));
	playgroundImages[2].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[2].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[2].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[2].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[2].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[2].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[2].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);
	playgroundImages[2].addStepColour(ColourHelpers::hslToRgb(ThreeFloats(Random::getGlobal().getRandomFloat(), 0.6f, 0.66f)), 0.01f, FancyImage::Smoothstep, 0.0f, true);

	addEntity();
}

//------------------------------------------------------------------------------
TitleScreen::~TitleScreen()
{

}

//------------------------------------------------------------------------------
void TitleScreen::update()
{
	float mag(joyPos.getMagnitude());
	const TwoFloats& drawSize(getApplication().getDrawingAreaSize());
	InteractFunc functions[] = {repelFunc,
								pauseFunc,
								rotateFunc};

	if(joyCount > 0)
		--joyCount;

	//if(!timeAttack || (!timerWait && (timer > 0.0f)))
	{
		if(entities.size() < 32)
		{
			--entityCount;
			if(!entityCount)
			{
				addEntity();
				entityCount = 180;
			}
		}

		if(mag > 0.5f)
		{
			TwoFloats tempJoy(joyPos);
			TwoFloats tempPos(mousePos);

			if(mag < 1.0f)
			{
				float scale((mag - 0.5f)/0.5f);

				tempJoy *= fakeExp(scale);
			}
			tempPos += tempJoy;

			if(tempPos.x < 0.0f)
				tempPos.x = 0.0f;
			else if(tempPos.x >= drawSize.x)
				tempPos.x = drawSize.x;

			if(tempPos.y < 0.0f)
				tempPos.y = 0.0f;
			else if(tempPos.y >= drawSize.y)
				tempPos.y = drawSize.y;

			mousePos = tempPos;
		}

		if(filteredPos.getDistanceSquared(mousePos) < (0.25f * 0.25f))
			filteredPos = mousePos;
		else
		{
			TwoFloats tempPos(filteredPos);

			tempPos += (mousePos - tempPos) * 0.033f;
			if((tempPos - filteredPos).getMagnitude() > 2.0f)
				tempPos = filteredPos + ((tempPos - filteredPos).normal() * 2.0f);

			filteredPos = tempPos;
		}

		for(unsigned int i=0;i<entities.size();++i)
		{
			TwoFloats effectorPos(-10000.0f);

			entities[i]->update(effectorPos, functions[imageIndex], drawSize);
		}
		for(unsigned int i=0;i<triangles.size();++i)
			triangles[i]->update(filteredPos, functions[imageIndex], drawSize);

		collisions();
	}

	for(int i=(explosions.size()-1);i>=0;--i)
	{
		explosions[i]->update();
		if(explosions[i]->getDead())
			explosions.erase(explosions.begin() + i);
	}

	highlights[0].update();
	highlights[1].update();

	clockImages[0].update();
	clockImages[1].update();

	playgroundImages[0].update();
	playgroundImages[1].update();
	playgroundImages[2].update();

	shakeIndex += 0.3f;
	if(shakeIndex > (2.0f * NIALL_PI))
		shakeIndex -= 2.0f * NIALL_PI;
	if(shakeAmp > 0.0f)
		shakeAmp -= 0.03f;

	if(!updateFromDraw)
	{
		if(!fadeTween.getFinished())
		{
			fadeTween.tick();

			if(fadeTween.getFinished())
			{
				if(fade >= 1.0f)
				{
					if(fadeTween.getDelta() > 0.01f)
						nextState(L"GamePlayState");
					else
						quit();
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void TitleScreen::draw(DrawerPtr& d, float timestepInterp)
{
	float shakeVal(0.0f);
	const std::wstring back[] = {L"BlueBack",
								 L"GreenBack",
								 L"RedBack"};
	const std::wstring front[] = {L"BlueFront",
								  L"GreenFront",
								  L"RedFront"};
	const TwoFloats& drawSize(d->getDrawingAreaSize());

	d->fillRect(0.0f, d->getDrawingAreaSize(), 1.0f);

	if(shakeAmp > 0.0f)
		shakeVal = ((sinf(shakeIndex) * 0.01f) + 0.01f) * shakeAmp;
	d->drawImage(d->getFrameBufferImage(),
				 TwoFloats(-(shakeVal * drawSize.x * 0.5f), 0.0f),
				 TwoFloats(1.0f + shakeVal, 1.0f));

	d->startFrameBufferDrawing();

	if(firstGo)
	{
		d->fillRect(0.0f, drawSize, 1.0f);
		firstGo = false;
	}

	for(unsigned int i=0;i<entities.size();++i)
		entities[i]->draw(d);
	for(unsigned int i=0;i<triangles.size();++i)
		triangles[i]->draw(d);

	updateFromDraw = true;
	update();
	updateFromDraw = false;

	for(unsigned int i=0;i<entities.size();++i)
		entities[i]->draw(d);
	for(unsigned int i=0;i<triangles.size();++i)
		triangles[i]->draw(d);

	d->endFrameBufferDrawing();

	for(unsigned int i=0;i<explosions.size();++i)
		explosions[i]->draw(d);

	highlights[0].draw(d);
	highlights[1].draw(d);

	clockImages[0].draw(d);
	clockImages[1].draw(d);

	playgroundImages[0].draw(d);
	playgroundImages[1].draw(d);
	playgroundImages[2].draw(d);

	d->drawText(L"Wake Up, Little One",
				L"BigFont",
				titleLayout,
				TwoFloats(0.0f, 128.0f),
				drawSize.x,
				TextLayout::Centre,
				1.0f,
				0.0f);

	d->drawText(L"Time Attack                Playground",
				L"BigFont",
				gamesLayout,
				TwoFloats(0.0f, (drawSize.y - 192.0f)),
				drawSize.x,
				TextLayout::Centre,
				1.0f,
				0.0f);
	d->drawText(L"Quit",
				L"BigFont",
				quitLayout,
				TwoFloats(0.0f, (drawSize.y - 64.0f)),
				drawSize.x,
				TextLayout::Centre,
				1.0f,
				0.0f);

	if(fade > 0.0f)
		d->fillRect(0.0f, drawSize, 1.0f, fade);
}

//------------------------------------------------------------------------------
void TitleScreen::keyDown(const std::wstring& action,
							const std::wstring& keyPressed,
							int keyModifier)
{
	bool newEntry(false);
	const TwoFloats& drawSize(getApplication().getDrawingAreaSize());

	if(action == L"Escape")
	{
		if((fadeTween.getEndVal() < 1.0f) && fadeTween.getFinished())
		{
			fadeTween.setDelta(0.01f);
			fadeTween.reset(1.0f);
			sounder->triggerFadeOut();
		}
		else
		{
			quit();
			return;
		}
	}
	else if(action == L"P1Left")
	{
		if(currentEntry > 0)
		{
			currentEntry = 0;
			newEntry = true;
		}
	}
	else if(action == L"P1Right")
	{
		if(currentEntry != 1)
		{
			currentEntry = 1;
			newEntry = true;
		}
	}
	else if(action == L"P1Down")
	{
		currentEntry = 2;
		newEntry = true;
	}
	else if(action == L"P1Up")
	{
		if(currentEntry = 2)
		{
			currentEntry = 0;
			newEntry = true;
		}
	}
	else
	{
		switch(currentEntry)
		{
			case 0:
				dynamic_pointer_cast<WakeUpGlobalData>(getApplication().getGlobalData())->setPlayground(false);
				fadeTween.setDelta(0.05f);
				break;
			case 1:
				dynamic_pointer_cast<WakeUpGlobalData>(getApplication().getGlobalData())->setPlayground(true);
				fadeTween.setDelta(0.05f);
				break;
			case 2:
				fadeTween.setDelta(0.01f);
				sounder->triggerFadeOut();
				break;
		}
		fadeTween.reset(1.0f);
	}

	if(newEntry)
	{
		switch(currentEntry)
		{
			case 0:
				highlights[0].tweenPosition(TwoFloats((drawSize.x * 0.5f) - 320.0f,
													  (drawSize.y * 0.5f)),
											0.05f,
											FancyImage::Smoothstep);
				highlights[1].tweenPosition(TwoFloats((drawSize.x * 0.5f) - 320.0f,
													  (drawSize.y * 0.5f)),
											0.05f,
											FancyImage::Smoothstep);

				clockImages[1].addStepRotation(2.0f * NIALL_PI, 0.001f, FancyImage::Linear, 0.0f, true, true);

				playgroundImages[1].setRotation(0.0f);
				break;
			case 1:
				highlights[0].tweenPosition(TwoFloats((drawSize.x * 0.5f) + 320.0f,
													  (drawSize.y * 0.5f)),
											0.05f,
											FancyImage::Smoothstep);
				highlights[1].tweenPosition(TwoFloats((drawSize.x * 0.5f) + 320.0f,
													  (drawSize.y * 0.5f)),
											0.05f,
											FancyImage::Smoothstep);

				playgroundImages[1].addStepRotation(-0.33f, 0.01f, FancyImage::Smoothstep, 0.0f, true);
				playgroundImages[1].addStepRotation(0.33f, 0.01f, FancyImage::Smoothstep, 0.0f, true);

				clockImages[1].setRotation(0.0f);
				break;
			case 2:
				highlights[0].tweenPosition(TwoFloats((drawSize.x * 0.5f),
													  (drawSize.y * 0.5f) + 272.0f),
											0.05f,
											FancyImage::Smoothstep);
				highlights[1].tweenPosition(TwoFloats((drawSize.x * 0.5f),
													  (drawSize.y * 0.5f) + 272.0f),
											0.05f,
											FancyImage::Smoothstep);

				clockImages[1].setRotation(0.0f);
				playgroundImages[1].setRotation(0.0f);
				break;
		}
	}
}

//------------------------------------------------------------------------------
void TitleScreen::keyUp(const std::wstring& action,
						  const std::wstring& keyPressed,
						  int keyModifier)
{
	
}

//------------------------------------------------------------------------------
void TitleScreen::mouseMove(const TwoFloats& pos)
{
	//mousePos = pos;
}

//------------------------------------------------------------------------------
void TitleScreen::joyMove(const std::wstring& action,
						  float value,
						  int joystick,
						  int axis)
{
	if(!joyCount)
	{
		if(fabsf(value) > 0.33f)
		{
			keyDown(action, L"", 0);
			joyCount = 40;
		}
	}
}

//------------------------------------------------------------------------------
void TitleScreen::repelFunc(TwoFloats& pos,
							  const TwoFloats& playerPos,
							  float rotDir)
{
	TwoFloats newVal(pos - ((playerPos -  pos) * 0.1f));

	if(newVal.getMagnitude() > 3.0f)
		newVal = pos + ((newVal - pos).normal() * 3.0f);

	pos = newVal;
}

//------------------------------------------------------------------------------
void TitleScreen::attractFunc(TwoFloats& pos,
							    const TwoFloats& playerPos,
							    float rotDir)
{
	TwoFloats newVal(pos + ((playerPos -  pos) * 0.05f));

	if(newVal.getMagnitude() > 3.0f)
		newVal = pos + ((newVal - pos).normal() * 3.0f);

	float dist(pos.getDistance(playerPos));
	if(dist > 128.0f)
		pos = newVal;
	else if(dist > 32.0f)
		pos += (newVal - pos) * fakeLog((dist-45.0f)/(128.0f-45.0f));
}

//------------------------------------------------------------------------------
void TitleScreen::rotateFunc(TwoFloats& pos,
							   const TwoFloats& playerPos,
							   float rotDir)
{
	float index(playerPos.getAngle(pos));
	float rad(pos.getDistance(playerPos));

	index += rotDir * 0.0266f;

	pos.x = playerPos.x + (sinf(index) * rad);
	pos.y = playerPos.y + (cosf(index) * rad);
}

//------------------------------------------------------------------------------
void TitleScreen::pauseFunc(TwoFloats& pos,
							  const TwoFloats& playerPos,
							  float rotDir)
{
	
}

//------------------------------------------------------------------------------
void TitleScreen::addEntity()
{
	const TwoFloats& drawSize(getApplication().getDrawingAreaSize());
	TwoFloats starts[] = {TwoFloats(-32.0f, 0.0f),
						  TwoFloats(0.0f, -32.0f),
						  TwoFloats(drawSize.x + 32.0f, 0.0f),
						  TwoFloats(0.0f, drawSize.y + 32.0f)};
	int index(Random::getGlobal().getRandomInt(4));

	TwoFloats pos(starts[index]);

	if(pos.x == 0.0f)
		pos.x = Random::getGlobal().getRandomFloat() * drawSize.x;
	else if(pos.y == 0.0f)
		pos.y = Random::getGlobal().getRandomFloat() * drawSize.y;

	ThreeFloats col(Random::getGlobal().getRandomFloat(), 1.0f, 0.8f);
	//ThreeFloats col(Random::getGlobal().getRandomFloat(), 1.0f, 0.5f);

	entities.push_back(shared_ptr<Entity>(new Entity(pos,
										  ColourHelpers::hslToRgb(col),
										  L"Entity",
										  28.0f,
										  drawSize,
										  sounder,
										  0.0f)));
	//sounder->triggerNewEntitySynth(col.x);
}

//------------------------------------------------------------------------------
void TitleScreen::collisions()
{
	const TwoFloats& drawSize(getApplication().getDrawingAreaSize());

	for(int i=(entities.size()-1);i>=0;--i)
	{
		const TwoFloats& pos1(entities[i]->getPosition());
		bool collided(false);

		for(int j=(i-1);j>=0;--j)
		{
			if(entities[i]->getImage() == entities[j]->getImage())
			{
				const TwoFloats& pos2(entities[j]->getPosition());
				float radSq(entities[i]->getRadius() - 6.0f);

				radSq *= 2.0f;
				radSq *= radSq;

				if(pos1.getDistanceSquared(pos2) < radSq)
				{
					int index(getNextImage(entities[i]->getImage()));

					if(index < 4)
					{
						TwoFloats pos((pos1 + pos2) * 0.5f);
						ThreeFloats col1(ColourHelpers::rgbToHsl(entities[i]->getColour()));
						ThreeFloats col2(ColourHelpers::rgbToHsl(entities[j]->getColour()));
						float r1(col1.x);
						float r2(col2.x);

						if(fabsf(r2 - r1) > 0.5f)
						{
							if(r2 > r1)
								r2 -= 1.0f;
							else if(r1 > r2)
								r1 -= 1.0f;
						}
						r1 = (r1 + r2) * 0.5f;
						if(r1 < 0.0f)
							r1 += 1.0f;
						ThreeFloats col(ColourHelpers::hslToRgb(ThreeFloats(r1, 1.0f, 0.8f)));
						//ThreeFloats col(ColourHelpers::hslToRgb(ThreeFloats(r1, 1.0f, 0.5f)));
						const std::wstring ims[] = {L"Entity",
												    L"Squidge",
												    L"Hexagon",
												    L"TriangleThing"};
						float rads[] = {28.0f,
										35.0f,
										40.0f,
										56.0f};

						entities.erase(entities.begin()+i);
						entities.erase(entities.begin()+j);

						if(index < 3)
						{
							entities.push_back(shared_ptr<Entity>(new Entity(pos,
																			 col,
																			 ims[index],
																			 rads[index],
																			 drawSize,
																			 sounder,
																			 (float)index/3.0f)));
							sounder->triggerNewEntitySynth(col.x);
						}
						else
						{
							triangles.push_back(shared_ptr<Entity>(new Entity(pos,
																			  col,
																			  ims[index],
																			  rads[index],
																			  drawSize,
																			  sounder,
																			  1.0f)));
							sounder->triggerNewTriangleSynth(col.x);

							//Add a new entity for every triangle created.
							addEntity();

							//Add the '+1' popup.
							shared_ptr<FancyImage> im(new FancyImage(L"PlusOne",
																	 TwoFloats(pos.x - 35.5f, pos.y - 106.0f)));

							im->tweenPosition(TwoFloats(pos.x - 35.5f, pos.y - 192.0f), 0.01f);
							im->tweenAlpha(0.0f, 0.01f, FancyImage::Exponential, 0.2f);
						}
						collided = true;
						
						explosions.push_back(shared_ptr<Explosion>(new Explosion(pos, col)));
						shakeAmp = 1.0f;

						break;
					}
				}
			}
		}
		if(collided)
			break;
	}
}

//------------------------------------------------------------------------------
int TitleScreen::getNextImage(const std::wstring& im)
{
	const std::wstring images[] = {L"Entity",
								   L"Squidge",
								   L"Hexagon",
								   L"TriangleThing"};
	int i;

	for(i=0;i<4;++i)
	{
		if(im == images[i])
		{
			++i;
			break;
		}
	}
	
	return i;
}
