//	TitleScreen.h - The title screen.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TITLESCREEN_H_
#define TITLESCREEN_H_

#include "drawing/TextLayout.h"
#include "drawing/FancyImage.h"
#include "GameCounter.h"
#include "GameState.h"
#include "Tweener.h"

#include <vector>

class WakeUpSounder;
class Explosion;
class Entity;

///	The main gameplay state.
class TitleScreen : public GameState
{
  public:
	///	Constructor.
	TitleScreen(Application& app, SounderPtr& sound);
	///	Destructor.
	~TitleScreen();

	///	Called frequently to update the state.
	void update();
	///	Called to draw the state.
	void draw(DrawerPtr& d, float timestepInterp);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);
	///	Called when a key up event is received.
	void keyUp(const std::wstring& action,
			   const std::wstring& keyPressed,
			   int keyModifier);

	///	Used to test connected line drawing.
	void mouseMove(const TwoFloats& pos);
	///	Used to move the thing.
	void joyMove(const std::wstring& action,
				 float value,
				 int joystick,
				 int axis);

	///	The Repel function.
	static void repelFunc(TwoFloats& pos,
						  const TwoFloats& playerPos,
						  float rotDir);
	///	The Attract function.
	static void attractFunc(TwoFloats& pos,
							const TwoFloats& playerPos,
							float rotDir);
	///	The Rotate function.
	static void rotateFunc(TwoFloats& pos,
						   const TwoFloats& playerPos,
						   float rotDir);
	///	The Rotate function.
	static void pauseFunc(TwoFloats& pos,
						  const TwoFloats& playerPos,
						  float rotDir);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"TitleScreen";};
	///	Used to construct instances of this state.
	static GameStateCreator<TitleScreen> creator;
  private:
	///	Helper method. Adds a random entity.
	void addEntity();
	///	Helper method. Handles Entity-Entity collisions.
	void collisions();
	///	Helper method. Returns the next image up for the passed-in image.
	int getNextImage(const std::wstring& im);

	///	Test.
	TwoFloats mousePos;
	///	Test.
	TwoFloats filteredPos;
	///	Test.
	int imageIndex;
	///	Test.
	TwoFloats joyPos;

	///	The entities in the game.
	std::vector<shared_ptr<Entity> > entities;
	///	The TriangleThing entities in the game.
	std::vector<shared_ptr<Entity> > triangles;
	///	Used to add entities.
	int entityCount;

	///	Any active explosions.
	std::vector<shared_ptr<Explosion> > explosions;

	///	Title text layout.
	TextLayout titleLayout;
	///	Games text layout.
	TextLayout gamesLayout;
	///	Quit text layout.
	TextLayout quitLayout;

	///	Which entry is currently selected.
	int currentEntry;
	///	Used to highlight the current entry.
	FancyImage highlights[2];

	///	The clock images.
	FancyImage clockImages[2];
	///	The playground images.
	FancyImage playgroundImages[3];

	///	Used to blank the background when the state starts.
	bool firstGo;
	///	Used to prevent false movements w/the joystick.
	int joyCount;

	///	Used to fade in/out.
	float fade;
	///	Used to fade in/out.
	Tweener<float> fadeTween;
	///	True if we're calling update() from draw().
	bool updateFromDraw;

	///	Used to 'shake' the screen.
	float shakeAmp;
	///	Used to 'shake' the screen.
	float shakeIndex;

	///	Our sounder.
	shared_ptr<WakeUpSounder> sounder;
};

#endif
