//	BitCrusher.cpp - Simple bit crusher/samplerate reducer.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "BitCrusher.h"

#include <cmath>

//------------------------------------------------------------------------------
BitCrusher::BitCrusher():
bitDepth(16.0f),
samplerate(1.0f),
mix(1.0f),
srIndex(0.0f),
crushStep(powf(0.5f, bitDepth)),
currentMix(1.0f),
mixTween(currentMix, currentMix, 0.00001f)
{
	lastSample[0] = 0.0f;
	lastSample[1] = 0.0f;

	mixTween.setIndex(1.0f);
}

//------------------------------------------------------------------------------
void BitCrusher::processAudio(float *buffer, int numSamples)
{
	for(int i=0;i<numSamples;++i)
	{
		srIndex += samplerate;
		if(srIndex >= 1.0f)
		{
			srIndex -= 1.0f;
			lastSample[0] = crushStep * floorf((buffer[(i*2)]/crushStep) + 0.5f);
			lastSample[1] = crushStep * floorf((buffer[(i*2)+1]/crushStep) + 0.5f);
		}
		buffer[(i*2)] *= 1.0f-currentMix;
		buffer[(i*2)+1] *= 1.0f-currentMix;
		buffer[(i*2)] += lastSample[0] * currentMix;
		buffer[(i*2)+1] += lastSample[1] * currentMix;

		if(!mixTween.getFinished())
			mixTween.tick();
	}
}

//------------------------------------------------------------------------------
void BitCrusher::setBitDepth(float val)
{
	crushStep = powf(0.5f, val);
}

//------------------------------------------------------------------------------
void BitCrusher::setSamplerate(float val)
{
	samplerate = val;
}

//------------------------------------------------------------------------------
void BitCrusher::setMix(float val, bool tween)
{
	if(tween)
		mixTween.setDelta(0.00001f);
	else
		mixTween.setDelta(0.5f);
	mixTween.reset(val);
}
