//	BitCrusher.h - Simple bit crusher/samplerate reducer.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BITCRUSHER_H_
#define BITCRUSHER_H_

#include "Tweener.h"

///	Simple bit crusher/samplerate reducer.
class BitCrusher
{
  public:
	///	Constructor.
	BitCrusher();

	///	Processes the input audio, writes the granulated output to the passed-in buffer.
	/*!
		\param buffer Contains the input audio, which should be overwritten
		with the plugin's output. NOTE: this is an INTERLEAVED stereo array.
		\param numSamples The number of samples in this buffer.

		Assumes a stereo input/output.
	 */
	void processAudio(float *buffer, int numSamples);

	///	Sets the bit depth (1->16).
	void setBitDepth(float val);
	///	Sets the samplerate (0->1; 0->samplrate).
	void setSamplerate(float val);
	///	Sets the mix.
	void setMix(float val, bool tween = false);
  private:
	///	The bit depth.
	float bitDepth;
	///	The samplerate.
	float samplerate;
	///	The effect's mix.
	float mix;

	///	The last audio sample from the samplerate reducer.
	float lastSample[2];
	///	Used to calculate the samplerate reduction.
	float srIndex;
	///	Used to calculate the bit crushing.
	float crushStep;
	///	Used to tween the mix parameter.
	float currentMix;
	///	Used to tween the mix parameter.
	Tweener<float> mixTween;
};

#endif
