//	Chorus.cpp - Simple chorus effect.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "MiscHelper.h"
#include "Chorus.h"

//------------------------------------------------------------------------------
Chorus::Chorus():
writePointer(0),
delay(1024.0f),
x0(0.0f),
x1(1.0f),
coeff(2.0f * sinf(NIALL_PI * 3.0f/44100.0f)),
depth(0.05f),
currentMix(0.0f),
mixTween(currentMix, currentMix, 0.00001f)
{
	for(int i=0;i<BufferSize;++i)
	{
		buffers[0][i] = 0.0f;
		buffers[1][i] = 0.0f;
	}

	mixTween.setIndex(1.0f);
}

//------------------------------------------------------------------------------
void Chorus::processAudio(float *buffer, int numSamples)
{
	for(int i=0;i<numSamples;++i)
	{
		//Write input into buffer.
		buffers[0][writePointer] = buffer[(i*2)];
		buffers[1][writePointer] = buffer[(i*2)+1];
		++writePointer;
		writePointer %= BufferSize;

		if(currentMix > 0.0f)
		{
			//Update LFO.
			x0 -= (coeff*x1);
			x1 += (coeff*x0);

			//Calculate the actual delay time for this sample.
			float delayVal((float)writePointer - delay);

			delayVal += (x0 * depth * (delay-1.0f));
			if(delayVal < 0.0f)
				delayVal += (float)BufferSize;
			else if(delayVal >= (float)BufferSize)
				delayVal -= (float)BufferSize; //Shouldn't ever get here, but just in case...

			int indices[4];

			indices[2] = (int)delayVal;
			indices[0] = indices[2] - 2;
			if(indices[0] < 0)
				indices[0] += BufferSize;
			indices[1] = indices[2] - 1;
			if(indices[1] < 0)
				indices[1] += BufferSize;
			indices[3] = indices[2] + 1;
			if(indices[3] >= BufferSize)
				indices[3] -= BufferSize;

			float fraction(delayVal - (float)((int)delayVal));

			//Write the delayed values on top of the input buffer.
			delayVal = bSplineInterp(buffers[0][indices[0]],
									 buffers[0][indices[1]],
									 buffers[0][indices[2]],
									 buffers[0][indices[3]],
									 fraction);
			buffer[(i*2)] += delayVal * currentMix;

			delayVal = bSplineInterp(buffers[1][indices[0]],
									 buffers[1][indices[1]],
									 buffers[1][indices[2]],
									 buffers[1][indices[3]],
									 fraction);
			buffer[(i*2)+1] += delayVal * currentMix;
		}

		if(!mixTween.getFinished())
			mixTween.tick();
	}
}

//------------------------------------------------------------------------------
void Chorus::setMix(float val)
{
	mixTween.reset(val);
}
