//	Flanger.h - Simple flanger effect.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef FLANGER_H_
#define FLANGER_H_

#include "Tweener.h"

///	Simple flanger effect.
class Flanger
{
  public:
	///	Constructor.
	Flanger();

	///	Processes the input audio, writes the granulated output to the passed-in buffer.
	/*!
		\param buffer Contains the input audio, which should be overwritten
		with the plugin's output. NOTE: this is an INTERLEAVED stereo array.
		\param numSamples The number of samples in this buffer.

		Assumes a stereo input/output.
	 */
	void processAudio(float *buffer, int numSamples);

	///	Sets the effect's mix value.
	void setMix(float val);
  private:
	///	B-spline interpolation helper method.
	inline float bSplineInterp(float x0,
							   float x1,
							   float x2,
							   float x3,
							   float val)
	{
		// 4-point, 3rd-order B-spline (x-form)
		float ym1py1 = x0+x2;
		float c0 = (1.0f/6.0f) * ym1py1 + (2.0f/3.0f) * x1;
		float c1 = (1.0f/2.0f) * (x2-x0);
		float c2 = (1.0f/2.0f) * ym1py1 - x1;
		float c3 = (1.0f/2.0f) * (x1-x2) + (1.0f/6.0f) * (x3-x0);
		return ((c3*val+c2)*val+c1)*val+c0;
	};

	///	Size of the delay buffer.
	enum
	{
		BufferSize = 44100
	};

	///	The two delay buffers.
	float buffers[2][BufferSize];
	///	The delay buffer write pointer.
	int writePointer;

	///	The delay time.
	float delay;
	///	Sine wave used to modulate the delay.
	float x0;
	///	Sine wave used to modulate the delay.
	float x1;
	///	Sine wave used to modulate the delay.
	float coeff;
	///	Depth of the modulation.
	float depth;

	///	Used to tween the mix parameter.
	float currentMix;
	///	Used to tween the mix parameter.
	Tweener<float> mixTween;
};

#endif
