#include "MoogFilter.h"

#include <limits>

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
MoogFilter::MoogFilter()
{
	res = 0.9;
	cutoff = 440.0;
fs=44100.0;

init();
}

MoogFilter::~MoogFilter()
{
}

void MoogFilter::init()
{
// initialize values
y1=y2=y3=y4=oldx=oldy1=oldy2=oldy3=0;
calc();
};

void MoogFilter::calc()
{
double f = (cutoff+cutoff) / fs; //[0 - 1]
p=f*(1.8-0.8*f);
k=p+p-1.0;

double t=(1.0-p)*1.386249;
double t2=12.0+t*t;
r = (res*(t2+6.0*t)/(t2-6.0*t));
};

float MoogFilter::process(float input)
{
const double invSix(1.0f/6.0f);

// process input
x = input - r*y4;

//This is a nasty (clicky) hack, but for some reason when the input signal's
//554.3652619538Hz the filter becomes unstable.  This is the only way I could
//think to fix it.  I know, I suck...
if(x == -std::numeric_limits<double>::infinity())
	y1=y2=y3=y4=oldx=oldy1=oldy2=oldy3=0;

//Four cascaded onepole filters (bilinear transform)
y1= x*p +  oldx*p - k*y1;
y2=y1*p + oldy1*p - k*y2;
y3=y2*p + oldy2*p - k*y3;
y4=y3*p + oldy3*p - k*y4;

if(y4 != y4)
	y1=y2=y3=y4=oldx=oldy1=oldy2=oldy3=0;

//Clipper band limited sigmoid
y4-=((y4*y4*y4)*invSix);
if(y4 == -std::numeric_limits<double>::infinity())
	y1=y2=y3=y4=oldx=oldy1=oldy2=oldy3=0;
if(y4 == std::numeric_limits<double>::infinity())
	y1=y2=y3=y4=oldx=oldy1=oldy2=oldy3=0;

oldx = x; oldy1 = y1; oldy2 = y2; oldy3 = y3;
return (float)y4;
}

float MoogFilter::getCutoff()
{ return (float)cutoff; }

void MoogFilter::setCutoff(float c)
{ cutoff=c; if(cutoff < 50.0) cutoff = 50.0; calc(); }

float MoogFilter::getRes()
{ return (float)res; }

void MoogFilter::setRes(float r)
{ res=r; calc(); }

void MoogFilter::setCutoffAndRes(float c, float r)
{ cutoff=c; if(cutoff < 50.0) cutoff = 50.0; res=r; calc(); }
