//	SineDrone.cpp - A simple sine-wave drone.
//  ----------------------------------------------------------------------------
//	This file is part of 'SoManyJaggedShards', an entry for 7DRL 2013.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SineDrone.h"

//------------------------------------------------------------------------------
SineDrone::SineDrone():
x0(0.0f),
x1(1.0f),
coeff(2.0f * sinf((440.0f/44100.0f) * NIALL_PI)),
amplitude(0.0f),
pan(0.5f),
dist(0.0f),
ampTween(amplitude, 0.0f, 0.00001f),
panTween(pan, 0.5f, 0.00001f)
{

}

//------------------------------------------------------------------------------
void SineDrone::getAudio(float *buffer, int numSamples)
{
	for(int i=0;i<numSamples;++i)
	{
		float sinVal;
		float distVal;

		x0 -= x1 * coeff;
		x1 += x0 * coeff;

		if(!ampTween.getFinished())
			ampTween.tick();

		sinVal = x0;
		distVal = tanhf(sinVal * NIALL_PI);
		distVal *= 0.5f + ((1.0f-dist) * 0.5f);

		sinVal += (distVal - sinVal) * dist;
		sinVal *= amplitude;

		buffer[(i*2)] += sinVal * (1.0f - pan);
		buffer[(i*2)+1] += sinVal * pan;
	}
}

//------------------------------------------------------------------------------
void SineDrone::setPitch(float val)
{
	coeff = 2.0f * sinf((val/44100.0f) * NIALL_PI);
}

//------------------------------------------------------------------------------
void SineDrone::setAmp(float val)
{
	ampTween.reset(val);
	if(amplitude <= 0.0f)
		ampTween.tick();
}

//------------------------------------------------------------------------------
void SineDrone::setPan(float val)
{
	panTween.reset(val);
}

//------------------------------------------------------------------------------
void SineDrone::setDist(float val)
{
	dist = val;
}
