//	SineDrone.h - A simple sine-wave drone.
//  ----------------------------------------------------------------------------
//	This file is part of 'SoManyJaggedShards', an entry for 7DRL 2013.
//	Copyright (C) 2014  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SINEDRONE_H_
#define SINEDRONE_H_

#include "Tweener.h"

///	A simple sine-wave drone.
class SineDrone
{
  public:
	///	Constructor.
	SineDrone();

	///	Returns the drone's generated audio.
	void getAudio(float *buffer, int numSamples);

	///	Sets the drone's pitch.
	void setPitch(float val);
	///	Sets the drone's amplitude.
	void setAmp(float val);
	///	Sets the drone's panning.
	void setPan(float val);
	///	Sets the drone's distortion.
	void setDist(float val);

	///	Returns the drone's amplitude.
	float getAmp() const {return amplitude;};
  private:
	///	Used to generate the drone.
	float x0;
	///	Used to generate the drone.
	float x1;
	///	Used to generate the drone.
	float coeff;

	///	The drone's amplitude.
	float amplitude;
	///	The drone's panning.
	float pan;
	///	How much the drone is distorted by.
	float dist;

	///	Tweens the drone's amplitude.
	LogTweener<float> ampTween;
	///	Tweens the drone's panning.
	Tweener<float> panTween;
};

#endif
